import logging
import random
from functools import wraps

import requests

from rollbot import as_plugin, RollbotFailure


def with_error_handle(f):
    @wraps(f)
    def wrapper(*a, **kw):
        try:
            return f(*a, **kw)
        except ConnectionError as _:
            return RollbotFailure.SERVICE_DOWN
    return wrapper


def fmt_list(nicks):
    if len(nicks) == 0:
        return "No one in teamspeak!"
    if len(nicks) == 1:
        return ("Only " if random.randint(0, 1) == 0 else "Oops all ") + nicks[0]
    return "I see the following people: " + ", ".join(nicks)


@as_plugin
@with_error_handle
def teamspeak(query_cfg: "teamspeak.query"):
    nicks = requests.get(query_cfg).json()["users"]
    return fmt_list(nicks)


@as_plugin
@with_error_handle
def teemspeem(query_cfg: "teamspeak.query"):
    nicks = requests.get(query_cfg).json()["users"]
    fmtd = fmt_list(nicks)
    for v in "aiou":
        fmtd = fmtd.replace(v, "e").replace(v.upper(), "E")
    return fmtd.replace("k", "m").replace("K", "M")


@as_plugin
@with_error_handle
def speamteek(query_cfg: "teamspeak.query"):
    nicks = requests.get(query_cfg).json()["users"]
    return fmt_list(nicks)[::-1]


@as_plugin
@with_error_handle
def teamscream(query_cfg: "teamspeak.query"):
    nicks = requests.get(query_cfg).json()["users"]
    return fmt_list(nicks).upper()


@as_plugin
@with_error_handle
def ts(query_cfg: "teamspeak.query"):
    nicks = requests.get(query_cfg).json()["users"]
    initials = [nick[0] for nick in nicks]
    if len(initials) == 0:
        return ""
    return "i c: " + ", ".join(initials)
